/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useContext } from "react";
import { Store } from 'Store'
import styled from 'styled-components'

const Unauthorized = React.lazy(() => import('./Unauthorized'));
const AppBarAdmin = React.lazy(() => import('./AppBar'));

const AdminTemplate = (props) => {

    const { state } = useContext(Store);

    const CheckUserPermission = () => {
        let canUserEdit = false;
    
        if (state.userIsLoggedIn) {
            const roles = [...state.currentUser.roles];
            for (let i = 0; i < roles.length; i++)
                if (roles[i].name === 'admin' || roles[i].name === 'editor')
                    canUserEdit = true;
        }
        else {
            canUserEdit = false;
        }
    
        return canUserEdit;
    }

    if (CheckUserPermission())
        return (
            <StyledDiv contrast={state.contrast}>
                <AppBarAdmin contrast={state.contrast}/>
                <div style={{margin: "20px"}}>
                    {props.inner}
                </div>
            </StyledDiv>
        )
    else
        return (
            <StyledDiv contrast={state.contrast}>
                <div>
                    <Unauthorized contrast={state.contrast}/>
                </div>
            </StyledDiv>
        )
}

const StyledDiv = styled.div`
    background-color: ${props => props.contrast === "" ? "white" : "black"};
    color: ${props => props.contrast === "" ? "#666" : "white"};
`

export default AdminTemplate;
