import React from 'react';
import PropTypes from "prop-types";
import Typography from "@material-ui/core/Typography";
import Box from "@material-ui/core/Box";
import AppBar from "@material-ui/core/AppBar";
import { Tab, Tabs } from "@material-ui/core";
import { Link } from 'react-router-dom'
import { TabsItens } from '../../Pages/AdminLabelTabs/LabelTabs'
import styled from "styled-components";

function TabPanel(props) {
    const { children, value, index, ...other } = props;

    return (
        <div
            role="tabpanel"
            hidden={value !== index}
            id={`nav-tabpanel-${index}`}
            aria-labelledby={`nav-tab-${index}`}
            {...other}
        >
            {value === index && (
                <Box p={3}>
                    <Typography>{children}</Typography>
                </Box>
            )}
        </div>
    );
}

TabPanel.propTypes = {
    children: PropTypes.node,
    index: PropTypes.any.isRequired,
    value: PropTypes.any.isRequired,
};

function a11yProps(index) {
    return {
        id: `nav-tab-${index}`,
        "aria-controls": `nav-tabpanel-${index}`,
    };
}
export default function AppBarAdmin(props) {
    const [value, setValue] = React.useState(0);

    const handleChange = (event, newValue) => {
        setValue(newValue);
    };

    return (
        <StyledAppBar position="sticky" contrast={props.contrast}>
            <StyledTabs
                variant="scrollable"
                scrollButtons="on"
                value={value}
                onChange={handleChange}
                aria-label="nav tabs example"
                TabIndicatorProps={{style: {background: props.contrast === "" ? "orange" : "yellow"}}}
            >
                {
                    TabsItens.map((label, index) => (
                        <StyledTab
                            contrast={props.contrast}
                            key={label.href}
                            label={label.label}
                            to={label.href}
                            icon={label.icon}
                            component={Link}
                            {...a11yProps(index)}
                        />
                    ))
                }
            </StyledTabs>
        </StyledAppBar>
    )
}

const StyledTab = styled(Tab)`
    svg {
        fill: ${props => props.contrast === "" ? "" : "white"} !important;
    }

    color: ${props => props.contrast === "" ? "" : "yellow"} !important;
    text-decoration: ${props => props.contrast === "" ? "" : "underline yellow"} !important;
`

const StyledAppBar = styled(AppBar)`
    background-color: ${props => props.contrast === "" ? "white" : "black"} !important;
    color: ${props => props.contrast === "" ? "#666" : "white"} !important;
    border-bottom: 1px solid ${props => props.contrast === "" ? "black" : "white"} !important;
    box-shadow: none !important;
`

const StyledTabs = styled(Tabs)`
    /*workaround because scrollButtons = on does not work*/
    .MuiTabScrollButton-root.Mui-disabled {
        opacity: 1 !important;
    }

    .MuiButtonBase-root.Mui-disabled {
        cursor: pointer !important;
        pointer-events: auto !important;
    }
`