/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useContext } from "react";
import moment from "moment";
// Maerial ui components
import Paper from "@material-ui/core/Paper";
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import EditRoundedIcon from "@material-ui/icons/EditRounded";
import { useStyles, StyledCard } from "../../Styles/DataCard";
import DeleteRoundedIcon from "@material-ui/icons/DeleteRounded";
//imports from local files
import { GetAData, DeleteFilter } from "../../../Filters";
import { Link, useHistory } from "react-router-dom";
import { apiDomain } from "../../../../env";
import { Grid } from "@material-ui/core";
import LoadingSpinner from "../../../../Components/LoadingSpinner";
import {
    getRequest,
    deleteRequest,
} from "../../../../Components/HelperFunctions/getAxiosConfig";
import SnackBar from "../../../../Components/SnackbarComponent";
import StyledButton from '../Button';

import { Store } from 'Store'

const CommunityQuestions = () => {

    const { state } = useContext(Store);

    const classes = useStyles();
    let history = useHistory();
    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get("learnObj");

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [item, setItem] = useState({}); //Necessary to consult the API, data

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };

    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    async function DeleteHandler() {
        deleteRequest(
            DeleteFilter("learning_objects", id),
            (data) => {
                if (data.errors)
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                else {
                    HandleSnack(
                        "O objeto educacional foi deletada com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    history.goBack();
                }
            },
            (error) => {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
            }
        );
    }

    useEffect(() => {
        getRequest(
            GetAData("learning_objects", id),
            (data, header) => {
                setItem(data);
                setIsLoaded(true);
                setError(false);
            },
            (error) => {
                setIsLoaded(true);
                setError(true);
            }
        );
    }, []);

    if (error) {
        return <div>Houve um erro</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />;
    } else {
        const DATA = [
            {
                subTitle: "ID",
                prop: item.id,
            },
            {
                subTitle: "Nome",
                prop: item.name,
            },
            {
                subTitle: "Descrição",
                prop: item.description,
            },
            {
                subTitle: "Dono(a)",
                prop: item.publisher.name,
            },
            {
                subTitle: "Autor(a)",
                prop: item.author,
            },
            {
                subTitle: "Score",
                prop: item.score,
            },
            {
                subTitle: "Status",
                prop: item.state,
            },
            {
                subTitle: "Criado em",
                prop: DisplayDate(item.created_at),
            },
            {
                subTitle: "Atualizado em",
                prop: DisplayDate(item.updated_at),
            },
            {
                subTitle: "Visualizações",
                prop: item.views_count,
            },
            {
                subTitle: "Likes",
                prop: item.likes_count,
            },
            {
                subTitle: "Avaliação média",
                prop: item.review_average,
            },
            {
                subTitle: "Likes",
                prop: item.likes_count,
            },
            {
                subTitle: "Linguagem",
                prop: (
                    <div
                        style={{
                            display: "flex",
                            flexDirection: "row",
                            flexWrap: "wrap",
                            justifyContent: "Space-between",
                        }}
                    >
                        {item.language.map((lan, index) => (
                            <Paper
                                elevation={3}
                                key={index}
                                style={{
                                    borderRadius: 18,
                                    padding: "0.8em",
                                    marginBottom: "1em",
                                    marginRight: "1em",
                                    backgroundColor: state.contrast === "" ? "#D3D3D3" : "black",
                                    color: state.contrast === "" ? "black" : "white",
                                    border: state.contrast === "" ? "" : "1px solid white",
                                    fontSize: 14,
                                }}
                            >
                                {lan.name}
                            </Paper>
                        ))}
                    </div>
                ),
            },
            {
                subTitle: "Tipo de recurso",
                prop: item.object_type,
            },
        ];

        return (
            <StyledCard contrast={state.contrast} variant="outlined">
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() =>
                        setSnackInfo({
                            message: "",
                            icon: "",
                            open: false,
                            color: "",
                        })
                    }
                />
                <CardContent>
                    <Grid
                        container
                        xs={12}
                        display="row"
                        justify="space-between"
                        alignItems="center"
                        alignContent="space-between"
                    >
                        <Grid item>
                            <Typography
                                className={classes.title}
                                gutterBottom
                            >
                                {item.name}
                            </Typography>
                        </Grid>
                        <Grid item>
                            <Link
                                style={{ textDecoration: "none" }}
                                to={`/admin/learningObjects`}
                            >
                                <StyledButton
                                    icon={<ListRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                    text="Listar"
                                />
                            </Link>
                            <Link
                                style={{ textDecoration: "none" }}
                                to={"/recurso/" + id}
                            >
                                <StyledButton
                                    icon={<VisibilityIcon />}
                                    color="primary"
                                    variant="outlined"
                                    text="Ver recurso"
                                />
                            </Link>

                            <Link
                                style={{ textDecoration: "none" }}
                                to={`/admin/learningObjectEdit?learnObj=${item.id}`}
                            >
                                <StyledButton
                                    icon={<EditRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                    text="Editar"
                                />
                            </Link>
                            <StyledButton
                                icon={<DeleteRoundedIcon />}
                                color="secondary"
                                variant="outlined"
                                onClick={DeleteHandler}
                                text="Deletar"
                            />
                        </Grid>
                    </Grid>
                    <div style={{ height: "1em" }} />
                    {item.thumbnail ? (
                        <div style={{ marginTop: "1em", marginBottom: "1em" }}>
                            <a target="_blank" rel="noreferrer" href={apiDomain + item.thumbnail}>
                                {" "}
                                <img src={apiDomain + item.thumbnail} alt="" />{" "}
                            </a>
                        </div>
                    ) : null}

                    {DATA.map((info, index) => (
                        <div className={classes.displayColumn} key={index}>
                            <Typography className={classes.subTitle}>
                                {info.subTitle}
                            </Typography>
                            <Typography>
                                {info.prop === null ? "Sem dados" : info.prop}
                            </Typography>
                        </div>
                    ))}
                </CardContent>
            </StyledCard>
        );
    }
};

export default CommunityQuestions;
