/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useContext } from "react";
import moment from 'moment';
// Maerial ui components
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import Grid from "@material-ui/core/Grid";
import { useStyles, StyledCard } from "../../Styles/DataCard";
// Icons
import EditRoundedIcon from "@material-ui/icons/EditRounded";
//imports from local files
import { GetAData } from "../../../Filters";
import { getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import { Link } from 'react-router-dom';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import StyledButton from '../Button';

import { Store } from 'Store'

const NoteCard = () => {

    const { state } = useContext(Store);

    const classes = useStyles();
    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get("id");

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [item, setItem] = useState({}); //Necessary to consult the API, data

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };


    useEffect(() => {
        getRequest(
            GetAData('scores', id),
            (data, header) => {
                setIsLoaded(true);
                setItem(data);
            },
            (error) => {
                setIsLoaded(true);
                setError(true);
            }
        )
    }, []);

    if (error) {
        return <div>Houve um erro</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {
        const DATA = [
            {
                subTitle: "ID",
                prop: item.id,
            },
            {
                subTitle: "Nome",
                prop: item.name,
            },
            {
                subTitle: "Código",
                prop: item.code,
            },
            {
                subTitle: "Peso",
                prop: item.weight,
            },
            {
                subTitle: "Criação",
                prop: DisplayDate(item.created_at),
            },
            {
                subTitle: "Atualizado",
                prop: DisplayDate(item.updated_at),
            },
        ]

        return (
            <StyledCard contrast={state.contrast}>
                <CardContent>
                    <Grid container xs={12} justify="space-between" alignContent="center" alignItems="center">
                        <Grid item>
                            <Typography className={classes.title} gutterBottom>
                                {item.name}
                            </Typography>
                        </Grid>
                        <Grid item>
                            <Link style={{ textDecoration: 'none' }} to={`/admin/noteVars`}>
                                <StyledButton
                                    icon={<ListRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                    text="Listar"
                                />
                            </Link>

                            <Link style={{ textDecoration: 'none' }} to={`/admin/noteVarEdit?id=${item.id}`}>
                                <StyledButton
                                    icon={<EditRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                    text="Editar"
                                />
                            </Link>
                        </Grid>
                    </Grid>
                    <div style={{height: "1em"}}/>
                    {DATA.map((info, index) => (
                        <div className={classes.displayColumn} key={index}>
                            <Typography className={classes.subTitle}>
                                {info.subTitle}
                            </Typography>
                            <Typography>
                                {info.prop === null ? "Sem dados" : info.prop}
                            </Typography>
                        </div>
                    ))}
                </CardContent>
            </StyledCard>
        );
    }
};

export default NoteCard;
