/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useContext } from 'react';
//imports material ui componets
import CardContent from "@material-ui/core/CardContent";
import CardAction from '@material-ui/core/CardActions';
import { Typography, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import MenuItem from "@material-ui/core/MenuItem";
import ListRoundedIcon from '@material-ui/icons/ListRounded';
//imports local files
import { postRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import SnackBar from '../../../../Components/SnackbarComponent';
import { StyledCard } from "../../../Components/Styles/DataCard";
import { Store } from 'Store'
import FormInput from "Components/FormInput.js"
import StyledButton from '../Button';
//router
import { Link } from 'react-router-dom';

const CreateQuestion = (props) => {

    const { state } = useContext(Store);

    const [status, setStatus] = useState('');
    const [description, setDescription] = useState('');

    const [isLoading, setIsLoading] = useState(false)

    // Handle error in Country 
    const [errorInDescription, setErrorInDescription] = useState({
        error: false,
        message: '',
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const DescriptionHandler = (e) => {
        if (errorInDescription.error) {
            setErrorInDescription({
                error: false,
                message: ''
            })
        }
        setDescription(e.target.value)
    }

    // verify if the given text is empty
    const isEmpty = (text) => {
        return text.length === 0 ? true : false;
    }

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    //Handle submit 
    async function onSubmit() {
        if (!isEmpty(description)) {
            setIsLoading(true)
            const api = '/questions'
            const body = {
                "question": {
                    'status': status,
                    'description': description,
                }
            }
            postRequest(
                api,
                body,
                (data) => {
                    if(data.errors)
                        HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                    else
                        HandleSnack('A pergunta foi criada com sucesso', true, 'success', '#228B22')
                    setIsLoading(false)
                },
                (error) => {
                    HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                    setIsLoading(false)
                }
            )
        }
        else
        {
            setErrorInDescription(
                {
                    error : true, 
                    message : "Você precisa preencher este campo!"
                }
            )
        }
    }

    //handle change of staus

    const handleChange = (e) => {
        const value = e.target.value;
        setStatus(value);
    };

    // Fields
    const fields = [
        {
            label: 'Descrição',
            value: description,
            required: true,
            error: errorInDescription.error,
            errorMessage: errorInDescription.message,
            onChange: (event) => DescriptionHandler(event)
        }
    ]

    const STATUS_OPTIONS = [
        {
            value: "active",
            label: "Ativo",
        },
        {
            value: "inactive",
            label: "Inativo",
        },
    ];

    return (
        <StyledCard contrast={state.contrast}>
            <SnackBar
                severity={snackInfo.icon}
                text={snackInfo.message}
                snackbarOpen={snackInfo.open}
                color={snackInfo.color}
                handleClose={() => setSnackInfo({
                    message: '',
                    icon: '',
                    open: false,
                    color: ''
                })}
            />
            <CardContent>
                <Grid container direction='row' justify='space-between' alignContent="center" alignItems="center" xs={12}>
                    <Grid item>
                        <Typography variant='h4'>
                            Nova pergunta
                        </Typography>
                    </Grid>
                    <Grid item>
                        <Link to={'/admin/Questions'} style={{ textDecoration: 'none' }}>
                            <StyledButton
                                onClick={props.BackToList}
                                icon={<ListRoundedIcon />}
                                variant='outlined'
                                color='primary'
                                text="Listar"
                            />
                        </Link>
                    </Grid>
                </Grid>

                <div style={{ height: '1em' }}></div>

                <form style={{ display: 'flex', flexDirection: 'column' }}>
                    <>
                        <FormInput
                            contrast={state.contrast}
                            selectable={true}
                            placeholder="Status"
                            value={status ? status : ""}
                            handleChange={handleChange}
                            items={STATUS_OPTIONS.map((option, index) => (
                                <MenuItem
                                    key={option.value}
                                    value={option.value}
                                    className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                                >
                                    {
                                        option.label
                                    }
                                </MenuItem>
                            ))}
                        />
                        {fields.map((field, index) => (
                            <FormInput
                                contrast={state.contrast}
                                key={index}
                                required={field.required}
                                error={field.error}
                                help={field.error ? field.errorMessage : ''}
                                style={{ width: '250px', marginBottom: '1em' }}
                                placeholder={field.label}
                                value={field.value}
                                handleChange={field.onChange}
                                inputType="search"
                                multi={true}
                            />
                        ))}
                    </>
                </form>
            </CardContent>
            <CardAction>
                <StyledButton
                    onClick={() => {
                        onSubmit();
                    }}
                    variant="contained"
                    color="primary"
                    disabled={isLoading}
                    icon={isLoading ? null : <AddRoundedIcon />}
                    text={isLoading ? <CircularProgress size={24} /> : 'Adicionar'}
                />
            </CardAction>
        </StyledCard>
    );
}

export default CreateQuestion;