/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useContext } from "react";
//material ui components
import MenuItem from "@material-ui/core/MenuItem";
import FormGroup from "@material-ui/core/FormGroup";
import FormControlLabel from "@material-ui/core/FormControlLabel";
import CircularProgress from "@material-ui/core/CircularProgress";
import Checkbox from "@material-ui/core/Checkbox";
import SendRoundedIcon from "@material-ui/icons/SendRounded";
import { makeStyles, withStyles } from '@material-ui/core/styles';
import Chip from '@material-ui/core/Chip';
import { yellow, blue } from "@material-ui/core/colors";
//imports from local files
import SnackBar from "../../../../Components/SnackbarComponent";
import { postRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import ClassicEditor from "@ckeditor/ckeditor5-build-classic"
import { CKEditor } from '@ckeditor/ckeditor5-react';
import { Store } from 'Store'
import FormInput from "Components/FormInput.js"
import StyledButton from '../Button';

const useStyles = makeStyles((theme) => ({
    root: {
        display: 'flex',
        justifyContent: 'center',
        flexWrap: 'wrap',
        listStyle: 'none',
        padding: theme.spacing(0.5),
        margin: 0,
    },
    chip: {
        margin: theme.spacing(0.5),
    },
}));

export const BlueCheckBox = withStyles({
    root: {
        color: blue[400],
        '&$checked': {
            color: blue[600],
        },
    },
    checked: {},
})((props) => <Checkbox color="default" {...props} />);

export const ContrastCheckBox = withStyles({
    root: {
        color: yellow[400],
        '&$checked': {
            color: yellow[600],
        },
    },
    checked: {},
})((props) => <Checkbox color="default" {...props} />);

const EmailInputs = (props) => {

    const { state } = useContext(Store);

    const classes = useStyles();
    const [option, setOption] = useState(props.email ? "Emails" : "All"); //labels of the text field 'to'
    const [isToEmails, setIsToEmails] = useState(props.email ? true : false)
    const [isToAll, setIsToAll] = useState(props.email ? false : true)
    const [isToRoles, setIsToRoles] = useState(false)

    // Capture th text insert by the user in the fields
    const [emails, setEmails] = useState(props.email ? props.email : "");
    const [emailsAdress, setEmailsAdress] = useState([]);
    const [subject, setSubject] = useState("");
    const [message, setMessage] = useState("");

    const [isSending, setIsSending] = useState(false);

    //Handle snack bar
    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    //Controls the state of error in the textfields
    const [errorInEmails, setErrorInEmail] = useState({
        error: false,
        arroba: false,
        message: "",
    });
    const [errorInSubject, setErrorInSubject] = useState({
        error: false,
        message: "",
    });

    const options = [
        {
            value: "All",
            label: "Todos os usuários",
        },
        {
            value: "Roles",
            label: "Roles/Permissões específicas",
        },
        {
            value: "Emails",
            label: "1 ou mais emails",
        },
    ];

    //Roles of permissions
    const [roles, setRoles] = useState([
        {
            label: "Editor",
            value: 7,
            isChecked: false,
        },
        {
            label: "Admin",
            value: 3,
            isChecked: false,
        },
        {
            label: "Curador",
            value: 4,
            isChecked: false,
        },
        {
            label: "Professor",
            value: 1,
            isChecked: false,
        },
        {
            label: "Submetedor",
            value: 8,
            isChecked: false,
        },
        {
            label: "Aluno",
            value: 2,
            isChecked: false,
        },
        {
            label: "Moderador",
            value: 5,
            isChecked: false,
        },
        {
            label: "Parceiro",
            value: 9,
            isChecked: false,
        },
        {
            label: "Supervisor",
            value: 6,
            isChecked: false,
        },
        {
            label: "Publicador",
            value: 10,
            isChecked: false,
        },
    ]);

    const handleChange = (e) => {
        const value = e.target.value;
        if (value === 'All') {
            setIsToAll(true)
            setIsToEmails(false)
            setIsToRoles(false)
        }
        if (value === 'Emails') {
            setIsToEmails(true)
            setIsToRoles(false)
            setIsToAll(false)
        }
        if (value === 'Roles') {
            setIsToRoles(true)
            setIsToAll(false)
            setIsToEmails(false)
        }
        setOption(value)
    };

    const handleChangeCheckBox = (i) => {
        const currState = [...roles];
        currState[i].isChecked = !currState[i].isChecked;
        setRoles(currState);
    };

    const EmailsHandler = (e) => {
        if (errorInEmails.error)
            setErrorInEmail({
                error: false,
                message: "",
            });
        setEmails(e.target.value)
    };

    const SubjectHandler = (e) => {
        if (errorInSubject.error) {
            setErrorInSubject({
                error: false,
                message: "",
            });
        }
        setSubject(e.target.value);
    };

    const OnKeyPressHandler = (key) => {
        if (key === 13) {
            if (!itsEmpty(emails)) {
                if (emails.includes("@")) {
                    const arr = [...emailsAdress];
                    arr.push(emails);
                    setEmails("");
                    setEmailsAdress(arr);
                } else {
                    setErrorInEmail({
                        error: true,
                        message: "Esse email não contém o caractere @",
                        arroba: false,
                    });
                }
            } else {
                setErrorInEmail({
                    error: true,
                    message: "Esse campo precisa ser preenchido",
                    arroba: false,
                });
            }
        }
    };

    //Delete emails adress from the list to be sent a message
    const HandleDelete = (i) => {
        const copyEmail = [...emailsAdress];
        copyEmail.splice(i, 1);
        setEmailsAdress(copyEmail);
    };

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    const CleanFields = () => {
        setMessage("");
        setSubject("");
        setEmailsAdress([]);
    };

    const itsEmpty = (obj) => {
        return obj.length === 0
    }

    const submitRequest = async () => {
        setIsSending(true);

        const rolesArr = [];
        const api = `/email`;
        const body = {
            "email": {
                "all_users": isToAll,
                "subject": subject,
                "body": message,
                "emails": isToEmails ? emailsAdress : [],
                "roles": isToRoles ? rolesArr : [],
            },
        };

        for (let index = 0; index < roles.length; index++) {
            const role = roles[index]
            if (role.isChecked)
                rolesArr.push(role.value)
        }

        if (isToRoles && !itsEmpty(rolesArr) && !itsEmpty(subject) && !itsEmpty(body))
            postRequest(
                api,
                body,
                (data, header) => {
                    HandleSnack(
                        "O email foi enviado com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    setIsSending(false);
                    CleanFields();
                },
                (error) => {
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                    setIsSending(false);
                    CleanFields();
                }
            )
        else if (isToEmails && !itsEmpty(emailsAdress) && !itsEmpty(subject) && !itsEmpty(body))
            postRequest(
                api,
                body,
                (data, header) => {
                    HandleSnack(
                        "O email foi enviado com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    setIsSending(false);
                    CleanFields();
                },
                (error) => {
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                    setIsSending(false);
                    CleanFields();
                }
            )
        else if (isToAll && !itsEmpty(emailsAdress) && !itsEmpty(subject) && !itsEmpty(body))
            postRequest(
                api,
                body,
                (data, header) => {
                    HandleSnack(
                        "O email foi enviado com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    setIsSending(false);
                    CleanFields();
                },
                (error) => {
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                    setIsSending(false);
                    CleanFields();
                }
            )
        else {
            HandleSnack(
                "Você precisa preencher todos os campos!",
                true,
                "warning",
                "#FFC125"
            );
            setIsSending(false);
        }
    };

    return (
        <div
            style={{
                width: "100%",
                display: "flex",
                flexDirection: "column",
                justifyContent: "space-around",
            }}
        >
            <form
                style={{
                    flex: 1,
                    display: "flex",
                    flexDirection: "column",
                }}
            >
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() =>
                        setSnackInfo({
                            message: "",
                            icon: "",
                            open: false,
                            color: "",
                        })
                    }
                />

                <FormInput
                    contrast={state.contrast}
                    id="outlined-input"
                    disableField={true}
                    placeholder="De *"
                    value="integrada.contato@mec.gov.br"
                /> 
                <div style={{ height: "1em" }} />
                <FormInput
                    contrast={state.contrast}
                    selectable={true}
                    placeholder="Para *"
                    value={option ? option : ""}
                    handleChange={handleChange}
                    help="Por favor, selecione uma das opções"
                    items={options.map((option, index) => (
                        <MenuItem
                            key={option.value}
                            value={option.value}
                            className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                        >
                            {option.label}
                        </MenuItem>
                    ))}
                />
                <div style={{ height: "1em" }} />
                {
                    isToRoles &&
                    <FormGroup style={{ marginBottom: "1em" }}>
                        {roles.map((role, index) => (
                            <FormControlLabel
                                key={index}
                                control={props.contrast === "" ? <BlueCheckBox checked={role.isChecked} onChange={() => handleChangeCheckBox(index)} name={role.label}/> : <ContrastCheckBox checked={role.isChecked} onChange={() => handleChangeCheckBox(index)} name={role.label}/>}
                                label={role.label}
                            />
                        ))}
                    </FormGroup>
                }
                {
                    isToEmails &&
                    <>
                        <div
                            style={{
                                display: "flex",
                                flexDirection: "row",
                                flexWrap: "wrap",
                                justifyContent: "Space-between",
                                marginBottom: "1em"
                            }}
                        >
                            {emailsAdress.map((email, index) => (
                                <li key={index} style={{ listStyleType: "none", marginBottom: "0.5em" }}>
                                    <Chip
                                        label={email}
                                        onDelete={() => HandleDelete(index)}
                                        color="primary"
                                        style={state.contrast === "" ? {backgroundColor: "#ddd", color: "black"} : {backgroundColor: "black", color: "white", border: "1px solid white"}}
                                        classes={classes.chip}
                                    />
                                </li>
                            ))}
                        </div>
                        <FormInput
                            contrast={state.contrast}
                            id="outlined-input"
                            error={errorInEmails.error}
                            help={errorInEmails.message}
                            value={emails}
                            onKeyPress={(key) => OnKeyPressHandler(key.which)}
                            handleChange={EmailsHandler}
                            placeholder="Digite um email por vez e pressione Enter"
                            style={{ marginBottom: "1em" }}
                        />
                    </>
                }
                <FormInput
                    contrast={state.contrast}
                    id="outlined-input"
                    error={errorInSubject.error}
                    help={errorInSubject.message}
                    value={subject}
                    onKeyPress={(key) => OnKeyPressHandler(key.which)}
                    handleChange={SubjectHandler}
                    placeholder="Digite o assunto do email"
                />
            </form >

            <div style={{ height: "1em" }} />

            <div style={{ flex: 1, color: "#666" }}>
                <CKEditor
                    editor={ClassicEditor}
                    data={message}
                    onBlur={(event, editor) => {
                        const data = editor.getData();
                        setMessage(data)
                    }}
                />
            </div>
            {/*
            <div style={{ fontSize: 14 }}>
                * Se você deseja enviar foto da sua máquina, é preciso <a href="mailto:name@email.com">Clicar aqui</a>
            </div>
            */}

            <div style={{ marginTop: 18 }}>
                <StyledButton
                    onClick={() => {
                        submitRequest();
                    }}
                    variant="contained"
                    disabled={isSending}
                    color="primary"
                    icon={<SendRoundedIcon />}
                    text={isSending ? <CircularProgress size={24} /> : "Enviar"}
                />
            </div>
        </div >
    );
};

export default EmailInputs;
