/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState, useContext } from "react";
import moment from "moment";
//imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import { Url, DeleteFilter } from "../../../Filters";
import AlertDialog from "../../../Components/Components/AlertDialog";
import LoadingSpinner from "../../../../Components/LoadingSpinner";
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import PageHeader, { StyledFilter } from "../../../Components/Components/PageHeader"
import FormInput from "Components/FormInput.js"
import StyledButton from '../../../Components/Components/Button';
//imports from material ui
import { makeStyles } from "@material-ui/core/styles";
import TableBody from "@material-ui/core/TableBody";
import PeopleRoundedIcon from "@material-ui/icons/PeopleRounded";
import TableCell from "@material-ui/core/TableCell";
import MenuItem from "@material-ui/core/MenuItem";
import TableRow from "@material-ui/core/TableRow";
import IconButton from "@material-ui/core/IconButton";
import { Button, Paper, Grid } from "@material-ui/core";
import CircularProgress from "@material-ui/core/CircularProgress";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import DeleteIcon from "@material-ui/icons/Delete";
import {
    deleteRequest,
    getRequest,
} from "../../../../Components/HelperFunctions/getAxiosConfig";
//routers
import { Link } from "react-router-dom";
import styled from 'styled-components'

import { Store } from 'Store'

export const useStylesCell = makeStyles((theme) => ({
    root: {
        color: props => props.contrast === "" ? "#666" : "white",
        border: props => props.contrast === "" ? "1px solid #666" : "1px solid white",
        "MuiTableRow-root": {
            backgroundColor: props => props.contrast === "" ? "#eee" : "black",
        }
    }
}));

export const useStylesRow = makeStyles((theme) => ({
    root: {
        "&:nth-of-type(odd)": {
            backgroundColor: props => props.contrast === "" ? "#eee" : "black",
        },
        backgroundColor: props => props.contrast === "" ? "" : "black",
        border: props => props.contrast === "" ? "1px solid #666" : "1px solid white",
    },
}));

const Collections = () => {

    const { state } = useContext(Store)

    const classesCell = useStylesCell(state);
    const classesRow = useStylesRow(state);

    const ADD_ONE_LENGHT = [""];
    const TOP_LABELS = [
        "NOME",
        "DESCRIÇÃO",
        "DONO(A)",
        "CRIAÇÃO",
        "ATUALIZAÇÃO",
        "PRIVACIDADE",
        "VISUALIZAR",
        "DELETAR",
    ]; //Labels from Table

    const WINDOW_WIDTH = window.innerWidth;

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [items, setItems] = useState([]); //Necessary to consult the API, data

    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false); //controlls the state of loadind more data

    const [showFilter, setShowFilter] = useState(false);
    const [valueOfSearch, setValueOfSearch] = useState("")
    const [search, setSearch] = useState("");
    const [currPage, setCurrPage] = useState(0)

    const [openAlertDialog, setOpenAlertDialog] = useState(false);
    const [deleteItem, setDeleteItem] = useState({}); //Delete Item
    const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);
    const [option, setOption] = useState(); //labels of the text field 'to'
    const [forceUpdate, setForceUpdate] = useState(false)

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    const privacyOptions = [
        { name: "private", value: "Privado" },
        { name: "public", value: "Público" }
    ];


    //handle snack info
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    //Defines which row must show the circular progress
    const HandleStateCircularProgress = (i) => {
        setIsLoadingToDelete(i);
    };

    //Called when user want to delete one institution
    async function DeleteHandler() {
        const id = deleteItem.id;
        HandleStateAlertDialog(null);
        deleteRequest(
            DeleteFilter("collections", id),
            (data) => {
                if (data.errors)
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                else {
                    HandleSnack(
                        "A Coleção foi deletada com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    setCurrPage(0)
                    HandleStateCircularProgress(null);
                    removeItemFromList(id);
                }
            },
            (error) => {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                HandleStateCircularProgress(null);
            }
        );
    }

    //Controlls the state of the Alert Dialog
    const HandleStateAlertDialog = (i) => {
        const obj = { ...items[i] };
        setDeleteItem(obj);
        setOpenAlertDialog(!openAlertDialog);
    };

    // handle change of privacy
    const handleChange = (e) => {
        setOption(e.target.value);
    };

    const HandleSearch = (e) => {
        setValueOfSearch(e.target.value)
    }

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
        .format("LLL")
        .toString();
    };

    const buildUrl = (privacyOpt, name) => {
        if (privacyOpt && name)
            return Url("collections", `"privacy" : "${privacyOpt}", "name" : "${name}"`, currPage, "DESC")
        else if (privacyOpt)
            return Url("collections", `"privacy" : "${privacyOpt}"`, currPage, "DESC")
        else if (name)
            return Url("collections", `"name" : "${name}"`, currPage, "DESC")
        else
            return Url("collections", "", currPage, "DESC")
    }

    const removeItemFromList = (itemId) => {
        let index = -1;
        for (let i = 0; i < items.length; i++) {
            const element = items[i];
            if (element.id === itemId) {
                index = i
                break
            }
        }
        if (index !== -1) {
            const cpyItems = [...items]
            cpyItems.splice(index, 1)
            setItems(cpyItems)
        }
    }

    useEffect(() => {
        if (currPage === 0)
            setIsLoaded(false)
        else
            setIsLoadingMoreItems(true)
        getRequest(
            buildUrl(option, search),
            (data, header) => {
                const arrData = [...data]
                if (arrData.length === 0) {
                    HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
                } else {
                    const arrItems = [...items]
                    if (currPage === 0) {
                        setItems(arrData.concat(ADD_ONE_LENGHT))
                    }
                    else {
                        arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
                        const arrResult = arrItems.concat(arrData)
                        setItems(arrResult.concat(ADD_ONE_LENGHT))
                    }
                }
                setIsLoaded(true)
                setIsLoadingMoreItems(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsLoadingMoreItems(false)
                setIsLoaded(true)
                setError(true)
            }
        )
    }, [currPage, search, option, forceUpdate])

    useEffect(() => {
        setCurrPage(0)
        setOption()
        setSearch("")
        setValueOfSearch("")
    }, [showFilter])

    return (
        <StyledContent contrast={state.contrast}>
        {
            error ? (
                <div>Error: {error.message}</div>
            )
            :
            (
                !isLoaded ? (
                    <LoadingSpinner text="Carregando..." />
                )
                :
                (
                    WINDOW_WIDTH <= 1024 ? (
                            <>
                            <SnackBar
                                severity={snackInfo.icon}
                                text={snackInfo.message}
                                snackbarOpen={snackInfo.open}
                                color={snackInfo.color}
                                handleClose={() =>
                                    setSnackInfo({
                                        message: "",
                                        icon: "",
                                        open: false,
                                        color: "",
                                    })
                                }
                            />
                            <AlertDialog
                                open={openAlertDialog}
                                OnDelete={DeleteHandler}
                                deleteItem={deleteItem}
                                HandleClose={() => {
                                    setOpenAlertDialog(false);
                                    HandleStateCircularProgress(null);
                                }}
                            />
                            <MobilePageHeader
                                title="Coleções"
                                actions={[
                                    {
                                        name: "Atualizar",
                                        isLoading: false,
                                        func: () => {
                                            setCurrPage(0)
                                            setForceUpdate(!forceUpdate)
                                        },
                                        icon: <UpdateRoundedIcon />
                                    },
                                    {
                                        name: "Filtrar",
                                        isLoading: false,
                                        func: () => {
                                            setShowFilter(!showFilter);
                                        },
                                        icon: <FilterListRoundedIcon />
                                    }
                                ]}
                            >
                                {showFilter ? (
                                <StyledFilter contrast={state.contrast}>
                                    <Grid
                                        container
                                        alignItems="center"
                                        alignContent="center"
                                        xs={12}
                                        direction="row"
                                        justify="space-between"
                                    >
                                        <Grid item>
                                            <FormInput
                                                contrast={state.contrast}
                                                selectable={true}
                                                placeholder="Filtro"
                                                value={option}
                                                handleChange={handleChange}
                                                help="Por favor, selecione uma das opções"
                                                items={privacyOptions.map((option, index) => (
                                                    <MenuItem
                                                        key={option.value}
                                                        value={option.name}
                                                        name={option.value}
                                                        className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                                                    >
                                                        {option.value}
                                                    </MenuItem>
                                                ))}
                                            />
                                        </Grid>
            
                                        <Grid item>
                                            <FormInput
                                                contrast={state.contrast}
                                                placeholder="Pesquisa"
                                                handleChange={(event) => HandleSearch(event)}
                                                value={valueOfSearch}
                                                onBlur={(event) => setSearch(event.target.value)}
                                                help="Retire o foco do campo de texto ao terminar de digitar"
                                            />
                                        </Grid>
                                    </Grid>
                                </StyledFilter>
                                ) : null}
                            </MobilePageHeader>
            
                            {items.map((row, index) =>
                                index === items.length - 1 ? (
                                    <StyledDivButton
                                        contrast={state.contrast}
                                        key={new Date().toISOString() + row.created_at}
                                    >
                                        <StyledLoadMoreButton
                                            contrast={state.contrast}
                                            color="primary"
                                            variant="text"
                                            // disabled={isLoadingMoreItems}
                                            startIcon={<AddRoundedIcon />}
                                            disabled={isLoadingMoreItems}
                                            onClick={() => {
                                                setCurrPage(currPage + 1)
                                            }}
                                        >
                                            {isLoadingMoreItems ? (
                                                <CircularProgress size={24} />
                                            ) : (
                                                "Carregar mais itens"
                                                )}
                                        </StyledLoadMoreButton>
                                    </StyledDivButton>
                                ) : (
                                    <>
                                    <MobileList
                                        contrast={state.contrast}
                                        key={new Date().toISOString() + row.created_at}
                                        title={row.name}
                                        subtitle={row.privacy}
                                        backColor={"#e81f4f"}
                                        avatar={<PeopleRoundedIcon />}
                                        href={`/admin/Collection?collection=${row.id}`}
                                        reset={() => {
            
                                        }}
                                        data={
                                        [
                                            {
                                                title: "ID",
                                                subtitle: row.id
                                            },
                                            {
                                                title: "Dono(a)",
                                                subtitle: row.owner ? row.owner.name : "Sem dados"
                                            },
                                            {
                                                title: "Criado em",
                                                subtitle: DisplayDate(row.created_at)
                                            },
                                            {
                                                title: "Atualizado em",
                                                subtitle: DisplayDate(row.updated_at)
                                            },
                                            {
                                                title: "Deletar",
                                                subtitle:
                                                    <StyledButton
                                                        variant="contained"
                                                        color="secondary"
                                                        onClick={() => {
                                                            HandleStateAlertDialog(index);
                                                            HandleStateCircularProgress(index);
                                                        }}
                                                        icon={<DeleteIcon />}
                                                        text="Deletar"
                                                    >
                                                    </StyledButton>
                                            }
                                        ]
                                        }
                                    />
                                    </>
                                )
                            )}
                            </>
                    )
                    :
                    (
                        <>
                        <SnackBar
                            severity={snackInfo.icon}
                            text={snackInfo.message}
                            snackbarOpen={snackInfo.open}
                            color={snackInfo.color}
                            handleClose={() =>
                                setSnackInfo({
                                    message: "",
                                    icon: "",
                                    open: false,
                                    color: "",
                                })
                            }
                        />
            
                        <PageHeader
                            title="Coleções"
                            actions={[
                                {
                                    name: "Atualizar",
                                    isLoading: false,
                                    func: () => {
                                        setCurrPage(0)
                                        setForceUpdate(!forceUpdate)
                                    },
                                    icon: <UpdateRoundedIcon />
                                },
                                {
                                    name: "Filtrar",
                                    isLoading: false,
                                    func: () => {
                                        setShowFilter(!showFilter);
                                    },
                                    icon: <FilterListRoundedIcon />
                                }
                            ]}
                        >
                            {showFilter ? (
                            <StyledFilter contrast={state.contrast}>
                                <Grid
                                    container
                                    alignItems="center"
                                    alignContent="center"
                                    xs={12}
                                    direction="row"
                                    justify="space-between"
                                >
                                    <Grid item>
                                        <FormInput
                                            contrast={state.contrast}
                                            selectable={true}
                                            placeholder="Filtro"
                                            value={option}
                                            handleChange={handleChange}
                                            help="Por favor, selecione uma das opções"
                                            items={privacyOptions.map((option, index) => (
                                                <MenuItem
                                                    key={option.value}
                                                    value={option.name}
                                                    name={option.value}
                                                    className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                                                >
                                                    {option.value}
                                                </MenuItem>
                                            ))}
                                        />
                                    </Grid>
            
                                    <Grid item>
                                        <FormInput
                                            contrast={state.contrast}
                                            placeholder="Pesquisa"
                                            handleChange={(event) => HandleSearch(event)}
                                            value={valueOfSearch}
                                            onBlur={(event) => setSearch(event.target.value)}
                                            help="Retire o foco do campo de texto ao terminar de digitar"
                                        />
                                    </Grid>
                                </Grid>
                            </StyledFilter>
                            ) : null}
                        </PageHeader>
            
                        <TableData top={TOP_LABELS}>
                            <TableBody>
                            {items.map((row, index) =>
                                index === items.length - 1 ? (
                                <TableRow className={classesRow.root} key={new Date().toISOString() + row.created_at}>
                                    <TableCell className={classesCell.root} colSpan={8}>
                                        <Button
                                            color="primary"
                                            variant="text"
                                            // disabled={isLoadingMoreItems}
                                            startIcon={<AddRoundedIcon />}
                                            disabled={isLoadingMoreItems}
                                            onClick={() => {
                                                setCurrPage(currPage + 1)
                                            }}
                                            style={state.contrast === "" ? {width: "100%"} : {width: "100%", color: "yellow", textDecoration: "underline yellow"}}
                                        >
                                            {isLoadingMoreItems ? (
                                                <CircularProgress size={24} />
                                            ) : (
                                                "Carregar mais itens"
                                            )}
                                        </Button>
                                    </TableCell>
                                </TableRow>
                                ) : (
                                    <TableRow className={classesRow.root} key={new Date().toISOString() + index}>
                                        <TableCell className={classesCell.root} component="th" scope="row">
                                            {row.name}
                                        </TableCell>
                                        <TableCell className={classesCell.root} align="right">
                                            <div
                                                dangerouslySetInnerHTML={{ __html: row.description }}
                                            ></div>
                                        </TableCell>
                                        <TableCell className={classesCell.root} align="right">
                                            {row.owner ? row.owner.name : "Sem dados"}
                                        </TableCell>
                                        <TableCell className={classesCell.root} align="right">
                                            {DisplayDate(row.created_at)}
                                        </TableCell>
                                        <TableCell className={classesCell.root} align="right">
                                            {DisplayDate(row.updated_at)}
                                        </TableCell>
                                        <TableCell className={classesCell.root} align="right">{row.privacy}</TableCell>
                                        <TableCell className={classesCell.root} align="right">
                                            <Link to={`/admin/Collection?collection=${row.id}`}>
                                                <IconButton>
                                                    <VisibilityIcon style={{ fill: state.contrast === "" ? "#00bcd4" : "yellow" }} />
                                                </IconButton>
                                            </Link>
                                        </TableCell>
                                        <TableCell className={classesCell.root} align="right">
                                            {isLoadingToDelete === index ? (
                                            <CircularProgress size={24} color="primary" />
                                            ) : (
                                                <IconButton
                                                    onClick={() => {
                                                        HandleStateAlertDialog(index);
                                                        HandleStateCircularProgress(index);
                                                    }}
                                                >
                                                    <DeleteIcon style={{ fill: state.contrast === "" ? "#FF0000" : "yellow" }} />
                                                </IconButton>
                                            )}
                                        </TableCell>
                                    </TableRow>
                                )
                            )}
                            </TableBody>
                        </TableData>
            
                        {/* This alert will be displayed if the user click to delete an institution */}
                        <AlertDialog
                            open={openAlertDialog}
                            OnDelete={DeleteHandler}
                            deleteItem={deleteItem}
                            HandleClose={() => {
                                setOpenAlertDialog(false);
                                HandleStateCircularProgress(null);
                            }}
                        />
                        </>
                    )
                )
            )
        }
        </StyledContent>
    );
};
export default Collections;

export const StyledLoadMoreButton = styled(Button)`
    background-color: ${props => props.contrast === "" ? "white" : "black"} !important;
    color: ${props => props.contrast === "" ? "" : "yellow"} !important;
    text-decoration: ${props => props.contrast === "" ? "" : "underline yellow"} !important;
`

export const StyledDivButton = styled(Paper)`
    width : 100%;
    display : flex; 
    justify-content : center; 
    align-items : center; 
    background-color: ${props => props.contrast === "" ? "white" : "black"} !important;
    color: ${props => props.contrast === "" ? "#666" : "white"} !important;
    border: 1px solid ${props => props.contrast === "" ? "#666" : "white"} !important;
`

const StyledContent = styled.div`
    background-color: ${props => props.contrast === "" ? "white" : "black"} !important;
    color: ${props => props.contrast === "" ? "#666" : "white"};
`