/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState, useContext } from "react";
import moment from 'moment';
//imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import { Url } from "../../../Filters";
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig';
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import PageHeader, { StyledFilter } from "../../../Components/Components/PageHeader"
import FormInput from "Components/FormInput.js"
import { Store } from 'Store'
import { StyledLoadMoreButton, StyledDivButton, useStylesCell, useStylesRow } from "./Collections.js";
import StyledButton from '../../../Components/Components/Button';
//imports from material ui
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import TableRow from "@material-ui/core/TableRow";
import IconButton from "@material-ui/core/IconButton";
import { Button, Grid } from "@material-ui/core";
import CircularProgress from "@material-ui/core/CircularProgress";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import EmailRoundedIcon from '@material-ui/icons/EmailRounded';
import ContactSupportRoundedIcon from "@material-ui/icons/ContactSupportRounded";
//routers
import { Link } from 'react-router-dom';

const CommunityQuestion = () => {

    const { state } = useContext(Store)

    const classesCell = useStylesCell(state);
    const classesRow = useStylesRow(state);

  const ADD_ONE_LENGHT = [""];
  const TOP_LABELS = [
    "ID",
    "DATA DE CONTATO",
    "NOME",
    "EMAIL",
    "MENSAGEM",
    "VISUALIZAR"
  ]; //Labels from Table
  const WINDOW_WIDTH = window.innerWidth

  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
  const [items, setItems] = useState([]); //Necessary to consult the API, data

  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false); //controlls the state of loadind more data
  const [currPage, setCurrPage] = useState(0)

  //Works with the filter
  const [invertList, setInvertList] = useState(false)
  const [showFilter, setShowFilter] = useState(false)
  const [valueOfMessageField, setValueOfMessageField] = useState("")
  const [message, setMessage] = useState("");
  const [valueOfEmailField, setValueOfEmailField] = useState("")
  const [email, setEmail] = useState("");
  const [valueOfNameField, setValueOfNameField] = useState("")
  const [name, setName] = useState("");
  const [forceUpdate, setForceUpdate] = useState(false)

  const [snackInfo, setSnackInfo] = useState({
    message: "",
    icon: "",
    open: false,
    color: "",
  });

  //handle with the message filter
  const valueOfMessageHandler = (e) => {
    setValueOfMessageField(e.target.value)
  }

  //handle with the email filter
  const valueOfEmailHandler = (e) => {
    setValueOfEmailField(e.target.value)
  }

  //handle with the name filter
  const valueOfNameHandler = (e) => {
    setValueOfNameField(e.target.value)
  }

  //handle snack info
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color,
    });
  };

  const DisplayDate = (date) => {
    const convertedData = moment.utc(date);
    return moment(convertedData)
      .format("LLL")
      .toString();
  };

  const cleanArrayAndInvert = () => {
    setInvertList(!invertList)
    setCurrPage(0)
    setItems([])
  }

  const buildUrl = (message, email, name, direction) => {
    if (message && email && name)
      return Url("contacts", `"message" : "${message}", "email" : "${email}", "name" : "${name}"`, currPage, direction)

    else if (message && name)
      return Url("contacts", `"message" : "${message}", "name" : "${name}"`, currPage, direction)
    else if (message && email)
      return Url("contacts", `"message" : "${message}", "email" : "${email}"`, currPage, direction)
    else if (name && email)
      return Url("contacts", `"name" : "${name}", "email" : "${email}"`, currPage, direction)

    else if (message)
      return Url("contacts", `"message" : "${message}"`, currPage, direction)
    else if (email)
      return Url("contacts", `"email" : "${email}"`, currPage, direction)
    else if (name)
      return Url("contacts", `"name" : "${name}"`, currPage, direction)
    else
      return Url("contacts", "", currPage, direction)
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      buildUrl(message, email, name, invertList ? "ASC" : "DESC"),
      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, message, email, name, invertList, forceUpdate])

  useEffect(() => {
    setCurrPage(0)
    setName("")
    setValueOfNameField("")
    setEmail("")
    setValueOfEmailField("")
    setMessage("")
    setValueOfMessageField("")
  }, [showFilter])

  if (error) {
    return <div>Error: {error.message}</div>;
  } else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {
    if (WINDOW_WIDTH <= 1200) {
      return (
        <>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() =>
              setSnackInfo({
                message: "",
                icon: "",
                open: false,
                color: "",
              })
            }
          />
          <MobilePageHeader
            title="Dúvidas da comunidade"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                  setForceUpdate(!forceUpdate)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Filtrar",
                isLoading: false,
                func: () => {
                  setShowFilter(!showFilter);
                },
                icon: <FilterListRoundedIcon />
              },
              {
                name: "Inverter lista de dados",
                isLoading: false,
                func: () => {
                  cleanArrayAndInvert()
                },
              }
            ]}
          >
            {showFilter ? (
                <StyledFilter contrast={state.contrast}>
              <Grid
                container
                direction="row"
                justify="space-between"
                alignItems="center"
                alignContent="flex-end"
                spacing={3}
                xs={12}
              >
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        placeholder="Email"
                        handleChange={valueOfEmailHandler}
                        value={valueOfEmailField}
                        onBlur={(event) => setEmail(event.target.value)}
                        help="Por favor, ao digitar o email que você quer filtar, retire o foco do campo de texto"
                    />
                </Grid>
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        placeholder="Mensagem"
                        handleChange={valueOfMessageHandler}
                        value={valueOfMessageField}
                        onBlur={(event) => setMessage(event.target.value)}
                        help="Por favor, ao digitar a mensagem que você quer filtar, retire o foco do campo de texto"
                    />
                </Grid>
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        placeholder="Nome"
                        handleChange={valueOfNameHandler}
                        value={valueOfNameField}
                        onBlur={(event) => setName(event.target.value)}
                        help="Por favor, ao digitar o nome que você quer filtar, retire o foco do campo de texto"
                    />
                </Grid>
              </Grid>
              </StyledFilter>
            ) : null}
          </MobilePageHeader>

          {items.map((row, index) =>
            index === items.length - 1 ? (
              <StyledDivButton
                contrast={state.contrast}
                key="Load more"
              >
                <StyledLoadMoreButton
                    contrast={state.contrast}
                  color="primary"
                  variant="text"
                  // disabled={isLoadingMoreItems}
                  startIcon={<AddRoundedIcon />}
                  disabled={isLoadingMoreItems}
                  onClick={() => {
                    setCurrPage(currPage + 1)
                  }}
                >
                  {isLoadingMoreItems ? (
                    <CircularProgress size={24} />
                  ) : (
                      "Carregar mais itens"
                    )}
                </StyledLoadMoreButton>
              </StyledDivButton>
            ) : (
                <>
                  <MobileList
                    contrast={state.contrast}
                    key={index}
                    title={row.name}
                    subtitle={row.id}
                    backColor={"#00bcd4"}
                    avatar={<ContactSupportRoundedIcon />}
                    href={`/admin/CommunityQuestion?question=${row.id}`}
                    reset={() => {
                    }}
                    data={
                      [
                        {
                          title: "Email",
                          subtitle:
                            row.email ?
                              <Link to={`/admin/sendEmail?email=${row.email}`} style={{ textDecoration: 'none' }}>
                                <StyledButton
                                  variant='text'
                                  color="primary"
                                  style={state.contrast === "" ? {} : {color: "yellow", textDecoration: "underline yellow"}}
                                  icon={<EmailRoundedIcon style={state.contrast === "" ? {color: "#666"} : {color: "white"}}/>}
                                  text={row.email}
                                />
                              </Link> : null

                        },
                        {
                          title: "Mensagem",
                          subtitle: row.message

                        },
                        {
                          title: "Criado em",
                          subtitle: DisplayDate(row.created_at)
                        },
                      ]
                    }
                  />
                </>
              )
          )}
        </>
      )
    }
    else {
      return <>
        <SnackBar
          severity={snackInfo.icon}
          text={snackInfo.message}
          snackbarOpen={snackInfo.open}
          color={snackInfo.color}
          handleClose={() =>
            setSnackInfo({
              message: "",
              icon: "",
              open: false,
              color: "",
            })
          }
        />

        <PageHeader
          title="Dúvidas da comunidade"
          actions={[
            {
              name: "Atualizar",
              isLoading: false,
              func: () => {
                setCurrPage(0)
                setForceUpdate(!forceUpdate)
              },
              icon: <UpdateRoundedIcon />
            },
            {
              name: "Filtrar",
              isLoading: false,
              func: () => {
                setShowFilter(!showFilter);
              },
              icon: <FilterListRoundedIcon />
            }
          ]}
        >
          {showFilter ? (
              <StyledFilter contrast={state.contrast}>
            <Grid
              container
              direction="row"
              justify="space-between"
              alignItems="center"
              alignContent="flex-end"
              spacing={3}
              xs={12}
            >
              <Grid item>
                <FormInput
                    contrast={state.contrast}
                    placeholder="Email"
                    handleChange={valueOfEmailHandler}
                    value={valueOfEmailField}
                    onBlur={(event) => setEmail(event.target.value)}
                    help="Por favor, ao digitar o email que você quer filtar, retire o foco do campo de texto"
                />
              </Grid>
              <Grid item>
                <FormInput
                    contrast={state.contrast}
                    placeholder="Mensagem"
                    handleChange={valueOfMessageHandler}
                    value={valueOfMessageField}
                    onBlur={(event) => setMessage(event.target.value)}
                    help="Por favor, ao digitar a mensagem que você quer filtar, retire o foco do campo de texto"
                />
              </Grid>
              <Grid item>
                <FormInput
                    contrast={state.contrast}
                    placeholder="Nome"
                    handleChange={valueOfNameHandler}
                    value={valueOfNameField}
                    onBlur={(event) => setName(event.target.value)}
                    help="Por favor, ao digitar o nome que você quer filtar, retire o foco do campo de texto"
                />
              </Grid>
            </Grid>
            </StyledFilter>
          ) : null}
        </PageHeader>

        <TableData top={TOP_LABELS} onIconPressed={cleanArrayAndInvert}>
          <TableBody>
            {items.map((row, index) =>
              index === items.length - 1 ? (
                <TableRow className={classesRow.root} key={row.created_at} style={{ padding: "1em" }}>
                  {/* Button to load more data */}
                  <TableCell className={classesCell.root} colspan={7}>
                    <Button
                      color="primary"
                      variant="text"
                      startIcon={<AddRoundedIcon />}
                      disabled={isLoadingMoreItems}
                      onClick={() => {
                        setCurrPage(currPage + 1)
                      }}
                      style={state.contrast === "" ? {width: "100%"} : {width: "100%", color: "yellow", textDecoration: "underline yellow"}}
                    >
                      {isLoadingMoreItems ? (
                        <CircularProgress size={24} />
                      ) : (
                          "Carregar mais itens"
                        )}
                    </Button>
                  </TableCell>
                </TableRow>
              ) : (
                  <TableRow className={classesRow.root} key={new Date().toISOString() + index}>
                    <TableCell className={classesCell.root} component="th" scope="row">
                      {row.id}
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                      {DisplayDate(row.created_at)}
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                      {row.name}
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                        {
                            row.email ?
                            <Link to={`/admin/sendEmail?email=${row.email}`} style={{ textDecoration: 'none' }}>
                                <StyledButton
                                    variant='text'
                                    color='primary'
                                    style={state.contrast === "" ? {} : {color: "yellow", textDecoration: "underline yellow"}}
                                    icon={<EmailRoundedIcon style={state.contrast === "" ? {color: "#666"} : {color: "white"}}/>}
                                    text={row.email}
                                />
                            </Link> : null
                        }
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                      {row.message}
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                      <Link to={`/admin/CommunityQuestion?question=${row.id}`}>
                        <IconButton>
                          <VisibilityIcon style={{ fill: state.contrast === "" ? "#00bcd4" : "yellow" }} />
                        </IconButton>
                      </Link>
                    </TableCell>
                  </TableRow>
                )
            )}
          </TableBody>
        </TableData>
      </>
    }
  }
}
export default CommunityQuestion;
