/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState, useContext } from "react";
import moment from "moment";
//imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import { Url } from "../../../Filters";
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import PageHeader, { StyledFilter } from "../../../Components/Components/PageHeader"
import FormInput from "Components/FormInput.js"
import { Store } from 'Store'
import StyledButton from '../../../Components/Components/Button';
import { StyledLoadMoreButton, StyledDivButton, useStylesCell, useStylesRow } from "./Collections.js";
//imports from material ui
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import MenuItem from "@material-ui/core/MenuItem";
import TableRow from "@material-ui/core/TableRow";
import IconButton from "@material-ui/core/IconButton";
import { Button, Paper, Grid } from "@material-ui/core";
import CircularProgress from "@material-ui/core/CircularProgress";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import LaunchRoundedIcon from "@material-ui/icons/LaunchRounded";
//routers
import { Link } from "react-router-dom";
import AnnouncementRoundedIcon from "@material-ui/icons/AnnouncementRounded";

const Complaints = () => {

    const { state } = useContext(Store)

    const classesCell = useStylesCell(state);
    const classesRow = useStylesRow(state);

  const WINDOW_WIDTH = window.innerWidth
  const ADD_ONE_LENGHT = [""];
  const PORTAL_MEC = "https://plataformaintegrada.mec.gov.br/";
  const TOP_LABELS = [
    "ESTADO DO RECURSO",
    "ID",
    "DESCRIÇÃO",
    "ID OBJETO",
    "TIPO",
    "DATA(MM/DD/YYYY)",
    "VISUALIZAR",
    "VISITAR",
  ]; //Labels from Table

  const [invertList, setInvertList] = useState(false)
  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
  const [items, setItems] = useState([]); //Necessary to consult the API, data

  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false); //controlls the state of loadind more data
  const [showFilter, setShowFilter] = useState(false);

  const [stateOption, setStateOption] = useState(0);
  const [complainOption, setComplainOption] = useState("");
  const [description, setDescription] = useState("");
  const [valueOfDescField, setValueOfDescField] = useState("")
  const [currPage, setCurrPage] = useState(0)
  const [forceUpdate, setForceUpdate] = useState(false)

  const [snackInfo, setSnackInfo] = useState({
    message: "",
    icon: "",
    open: false,
    color: "",
  });

  const stateOptions = [
    { name: 0, value: "Pendente" },
    { name: 1, value: "Ativo" },
    { name: 2, value: "Removido" },
  ];

  const ComplaintReasons = [
    { name: 1, value: "Viola direitos autorais" },
    { name: 2, value: "Conteúdo ofensivo/abusivo" },
    { name: 3, value: "Conta falsa" },
    { name: 4, value: "Spam" },
    { name: 5, value: "Descrição diverge do conteúdo" },
  ];

  //handle snack info
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color,
    });
  };

  const handleChangeState = (e, type) => {
    const value = e.target.value;
    setStateOption(value);
  };

  const handleChangeComplain = (e, type) => {
    const value = e.target.value;
    setComplainOption(value);
  };

  const DescriptionHandler = (e) => {
    setValueOfDescField(e.target.value)
  }

  const convertToLink = (type, id) => {
    switch (type) {
      case "LearningObject":
        return `recurso?id=${id}/`;
      case "User":
        return `usuario-publico/${id}/`;
      default:
        return "";
    }
  };

  const ComplaintStatus = (status, type) => {
    switch (status) {
      case "accepted":
        return (
          <Paper
            style={{
              textAlign: "center",
              padding: "0.5em",
              backgroundColor: state.contrast === "" ? "#FA8072" : "black",
              border: state.contrast === "" ? "" : "1px solid white",
              fontWeight: "500",
              color: "#FFFAFA",
            }}
          >
            {
              type === "User" ? "BLOQUEADO" : "REMOVIDO"
            }
          </Paper>
        );
      case "complained":
        return (
          <Paper
            style={{
              textAlign: "center",
              padding: "0.5em",
              backgroundColor: state.contrast === "" ? "#FF8C00" : "black",
              border: state.contrast === "" ? "" : "1px solid white",
              fontWeight: "500",
              color: "#FFFAFA",
            }}
          >
            PENDENTE
          </Paper>
        );
      case "rejected":
        return (
          <Paper
            style={{
              textAlign: "center",
              padding: "0.5em",
              backgroundColor: state.contrast === "" ? "#228B22" : "black",
              border: state.contrast === "" ? "" : "1px solid white",
              fontWeight: "500",
              color: "#FFFAFA",
            }}
          >
            REJEITADO
          </Paper>
        );
      default:
        return "NOTHING";
    }
  };

  const DisplayDate = (date) => {
    const convertedData = moment.utc(date);
    return moment(convertedData)
      .format("LLL")
      .toString();
  };

  const cleanArrayAndInvert = () => {
    setInvertList(!invertList)
    setCurrPage(0)
    setItems([])
  }

  const buildUrl = (complainOpt, state, description, direction) => {
    if (complainOpt && (state >= 0 && state <= 2) && description)
      return Url("complaints", `"state" : ${state}, "complaint_reason_id" : ${complainOpt}, "description" : "${description}"`, currPage, direction)

    else if (complainOpt && description)
      return Url("complaints", `"complaint_reason_id" : ${complainOpt}, "description" : "${description}"`, currPage, direction)
    else if (complainOpt && (state >= 0 && state <= 2))
      return Url("complaints", `"complaint_reason_id" : ${complainOpt}, "state" : ${state}`, currPage, direction)
    else if (description && (state >= 0 && state <= 2))
      return Url("complaints", `"description" : "${description}", "state" : ${state}`, currPage, direction)

    else if (complainOpt)
      return Url("complaints", `"complaint_reason_id" : ${complainOpt}`, currPage, direction)
    else if (state >= 0 && state <= 2)
      return Url("complaints", `"state" : ${state}`, currPage, direction)
    else if (description)
      return Url("complaints", `"description" : ${description}`, currPage, direction)
    else
      return Url("complaints", "", currPage, direction)
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      buildUrl(complainOption, stateOption, description, invertList ? "ASC" : "DESC"),
      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, complainOption, stateOption, description, invertList, forceUpdate])

  useEffect(() => {
    setComplainOption()
    setDescription("")
    setValueOfDescField("")
    setStateOption(0)
    setCurrPage(0)
  }, [showFilter])

  if (error) {
    return <div>Error: {error.message}</div>;
  } else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {
    if (WINDOW_WIDTH <= 994) {
      return (
        <>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() =>
              setSnackInfo({
                message: "",
                icon: "",
                open: false,
                color: "",
              })
            }
          />
          <MobilePageHeader
            title="Denúncias"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                  setForceUpdate(!forceUpdate)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Filtrar",
                isLoading: false,
                func: () => {
                  setShowFilter(!showFilter);
                },
                icon: <FilterListRoundedIcon />
              },
              {
                name: "Inverter lista",
                isLoading: false,
                func: () => {
                  cleanArrayAndInvert()
                },
              }
            ]}
          >
            {showFilter ? (
                <StyledFilter contrast={state.contrast}>
              <Grid
                container
                direction="row"
                justify="space-between"
                alignItems="center"
                alignContent="flex-end"
                spacing={3}
                xs={12}
              >
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        selectable={true}
                        placeholder="Motivo"
                        value={complainOption}
                        handleChange={handleChangeComplain}
                        help="Por favor, selecione uma das opções"
                        items={ComplaintReasons.map((option, index) => (
                            <MenuItem
                                key={option.value}
                                value={option.name}
                                name={option.value}
                                className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                            >
                                {option.value}
                            </MenuItem>
                        ))}
                    />
                </Grid>
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        selectable={true}
                        placeholder="Estado"
                        value={stateOption}
                        handleChange={handleChangeState}
                        help="Por favor, selecione uma das opções"
                        items={stateOptions.map((option, index) => (
                            <MenuItem
                                key={option.value}
                                value={option.name}
                                name={option.value}
                                className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                            >
                                {option.value}
                            </MenuItem>
                        ))}
                    />
                </Grid>
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        placeholder="Descrição"
                        handleChange={DescriptionHandler}
                        value={valueOfDescField}
                        onBlur={(event) => setDescription(event.target.value)}
                        help="Ao digitar a descrição, retire o foco do campo de texto"
                    />
                </Grid>
              </Grid>
              </StyledFilter>
            ) : null}
          </MobilePageHeader>

          {items.map((row, index) =>
            index === items.length - 1 ? (
              <StyledDivButton
                contrast={state.contrast}
                key={new Date().toISOString() + row.created_at}
              >
                <StyledLoadMoreButton
                  contrast={state.contrast}
                  color="primary"
                  variant="text"
                  // disabled={isLoadingMoreItems}
                  startIcon={<AddRoundedIcon />}
                  disabled={isLoadingMoreItems}
                  onClick={() => {
                    setCurrPage(currPage + 1)
                  }}
                >
                  {isLoadingMoreItems ? (
                    <CircularProgress size={24} />
                  ) : (
                      "Carregar mais itens"
                    )}
                </StyledLoadMoreButton>
              </StyledDivButton>
            ) : (
                <>
                  <MobileList
                    contrast={state.contrast}
                    key={new Date().toISOString() + row.created_at}
                    title={row.complainable_type}
                    subtitle={row.id}
                    backColor={"#673ab7"}
                    avatar={<AnnouncementRoundedIcon />}
                    href={`/admin/complaint?id=${row.id}`}
                    reset={() => {

                    }}
                    data={
                      [
                        {
                          title: "ID do objeto",
                          subtitle: row.complainable_id

                        },
                        {
                          title: "Criado em",
                          subtitle: DisplayDate(row.created_at)
                        },
                        {
                          title: "Descrição",
                          subtitle: row.description
                        },
                        {
                          title: "Estado",
                          subtitle: ComplaintStatus(row.state, row.complainable_type)
                        }
                      ]
                    }
                  />
                </>
              )
          )}
        </>
      );
    }
    else {
      return (
        <>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() =>
              setSnackInfo({
                message: "",
                icon: "",
                open: false,
                color: "",
              })
            }
          />

          <PageHeader
            title="Denúncias"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                  setForceUpdate(!forceUpdate)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Filtrar",
                isLoading: false,
                func: () => {
                  setShowFilter(!showFilter);
                },
                icon: <FilterListRoundedIcon />
              }
            ]}
          >
            {showFilter ? (
                <StyledFilter contrast={state.contrast}>
              <Grid
                container
                direction="row"
                justify="space-between"
                alignItems="center"
                alignContent="flex-end"
                spacing={3}
                xs={12}
              >
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        selectable={true}
                        placeholder="Motivo"
                        value={complainOption}
                        handleChange={handleChangeComplain}
                        help="Por favor, selecione uma das opções"
                        items={ComplaintReasons.map((option, index) => (
                            <MenuItem
                                key={option.value}
                                value={option.name}
                                name={option.value}
                                className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                            >
                                {option.value}
                            </MenuItem>
                        ))}
                    />
                </Grid>
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        selectable={true}
                        placeholder="Estado"
                        value={stateOption}
                        handleChange={handleChangeState}
                        help="Por favor, selecione uma das opções"
                        items={stateOptions.map((option, index) => (
                            <MenuItem
                                key={option.value}
                                value={option.name}
                                name={option.value}
                                className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                            >
                                {option.value}
                            </MenuItem>
                        ))}
                    />
                </Grid>
                <Grid item>
                    <FormInput
                        contrast={state.contrast}
                        placeholder="Descrição"
                        handleChange={DescriptionHandler}
                        value={valueOfDescField}
                        onBlur={(event) => setDescription(event.target.value)}
                        help="Ao digitar a descrição, retire o foco do campo de texto"
                    />
                </Grid>
              </Grid>
              </StyledFilter>
            ) : null}
          </PageHeader>

          <Grid xs={12} container>
            <TableData
              top={TOP_LABELS}
              onIconPressed={cleanArrayAndInvert}
            >
              <TableBody>
                {items.map((row, index) =>
                  index === items.length - 1 ? (
                    <TableRow className={classesRow.root} key={new Date().toISOString() + row.created_at}>
                      {/* Button to load more data */}
                      <TableCell className={classesCell.root} colspan={8}>
                        <Button
                          color="primary"
                          variant="text"
                          // disabled={isLoadingMoreItems}
                          startIcon={<AddRoundedIcon />}
                          disabled={isLoadingMoreItems}
                          onClick={() => {
                            setCurrPage(currPage + 1)
                          }}
                          style={state.contrast === "" ? {width: "100%"} : {width: "100%", color: "yellow", textDecoration: "underline yellow"}}
                        >
                          {isLoadingMoreItems ? (
                            <CircularProgress size={24} />
                          ) : (
                              "Carregar mais itens"
                            )}
                        </Button>
                      </TableCell>
                    </TableRow>
                  ) : (
                      <TableRow className={classesRow.root}
                        key={new Date().toISOString() + index}
                        style={{ flex: 1, width: "100%" }}
                      >
                        <TableCell className={classesCell.root} component="th" scope="row">
                          {ComplaintStatus(row.state, row.complainable_type)}
                        </TableCell>
                        <TableCell className={classesCell.root} align="right">{row.id}</TableCell>
                        <TableCell className={classesCell.root} align="right">
                          {row.description}
                        </TableCell>
                        <TableCell className={classesCell.root} align="right">
                          {row.complainable_id}
                        </TableCell>
                        <TableCell className={classesCell.root} align="right">
                          {row.complainable_type}
                        </TableCell>
                        <TableCell className={classesCell.root} align="right">
                          {DisplayDate(row.created_at)}
                        </TableCell>
                        <TableCell className={classesCell.root} align="right">
                          <Link to={`/admin/complaint?id=${row.id}`}>
                            <IconButton
                              onClick={() => {
                              }}
                            >
                              <VisibilityIcon style={{ fill: state.contrast === "" ? "#00bcd4" : "yellow" }} />
                            </IconButton>
                          </Link>
                        </TableCell>
                        <TableCell className={classesCell.root} align="right">
                            <a href={PORTAL_MEC +
                                convertToLink(
                                  row.complainable_type,
                                  row.complainable_id
                                )}
                            >
                                <StyledButton
                                    variant="text"
                                    secondary={true}
                                    icon={
                                        <LaunchRoundedIcon style={{ fill: "#FA8072" }} />
                                    }
                                    text="MEC RED"
                                />
                            </a>
                        </TableCell>
                      </TableRow>
                    )
                )}
              </TableBody>
            </TableData>
          </Grid>
        </>
      );
    }
  }
};
export default Complaints;