/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React from 'react'
import styled from 'styled-components'

function GetEmbeddedLink(link) {
    var embed = undefined
    var link_id = undefined
    if (link.indexOf("youtube") !== -1) {  //plain youtebe.com/ link
        if (link.indexOf("embed/") !== -1) { //if it's already an embedded link, return it
            return link
        }
        link = link.split("&")[0] //else remove features and other queries
        link_id = link.split("v=")[1] //get video id
        embed = "https://www.youtube.com/embed/" + link_id; //create embedded link
    }
    else if (link.indexOf("youtu.be") !== -1) { //if it's a youtu.be link
        link = link.split("&")[0].split("?")[0] //remove queries and features if existent
        link_id = link.split(".be/")[1] //get video id
        embed = "https://www.youtube.com/embed/" + link_id; //create embedded link
    }
    else if (link.indexOf("vimeo") !== -1) {    //if the 13th character = o (vimeo videos)
        link_id = link.split("?")[0].split("/")
        embed = "https://player.vimeo.com/video/" + link_id.pop();    //Add vimeo link before key #
    }
    return embed
}


export default function VideoPlayer(props) {

    return (
        <>
            {
                props.urlVerified ?
                    (
                        <VideoContainer>
                            <iframe
                                title="Video Player"
                                src={GetEmbeddedLink(props.link)}
                                frameBorder="0" allowFullScreen className="video"
                            />
                        </VideoContainer>
                    )
                    :
                    (
                        props.videoType === "video/mp4" ?
                            <VideoContainer>
                                <video controls className="video">
                                    <source src={props.videoUrl} type="video/webm" />
                                    <source src={props.videoUrl} type="video/mp4" />
                                </video>
                            </VideoContainer>
                            :
                            <ErrorParagraph contrast={props.contrast}>
                                Seu navegador não permite a exibição deste vídeo. É necessário baixar o vídeo para poder visualizá-lo.
                            </ErrorParagraph>
                    )
            }
        </>
    )
}

const VideoContainer = styled.div`
    position : relative;
    width : 100%;
    height : 0;
    padding-bottom : 56.25%;

    .video {
        width : 100%;
        height : 100%;
        position : absolute;
        top : 0;
        left : 0;
    }
`

const ErrorParagraph = styled.p`
    text-align: center;
    color: ${props => props.contrast === "" ? "#666" : "white"};
`
