/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React from 'react'
import Grid from '@material-ui/core/Grid';
import NoContent from './NoContent.js'
import ResourceCardFunction from '../../ResourceCardFunction.js'
import Title from './PanelTitle.js'
import { WhiteContainer, StyledGrid } from '../StyledComponents.js'
import { ButtonsAreaRecurso } from './ButtonsArea'
import LoadingSpinner from '../../LoadingSpinner.js';

export default function Template(props) {
    return (
        <WhiteContainer contrast={props.contrast}>
            <Title
                contrast={props.contrast}
                title={props.titleText}
                length={props.end}
            />

            {
                props.error ?
                    <p
                        className="errorText"
                    >
                        Erro ao tentar obter os dados
                    </p>
                    :
                    props.length === 0 ?
                        (
                            [
                                <NoContent contrast={props.contrast} text={props.noContentText} />
                            ]
                        )
                        :
                        (
                            [
                                <React.Fragment>
                                    <StyledGrid container spacing={1} justify="center" alignItems="center" style={{ paddingLeft: "30px", paddingRight: "15px" }}>
                                        {
                                            props.sliceArr.map((card) =>
                                                <Grid item xs={12} sm={6} md={'auto'} lg={3} key={card.id}>
                                                    <ResourceCardFunction
                                                        contrast={props.contrast}
                                                        avatar={card.submitter.avatar}
                                                        id={card.learning_object.id}
                                                        thumbnail={card.learning_object.thumbnail}
                                                        type={card.learning_object.object_type ? card.learning_object.object_type : "Outros"}
                                                        title={card.learning_object.name}
                                                        published={false}
                                                        likeCount={card.learning_object.likes_count}
                                                        liked={card.learning_object.liked}
                                                        rating={card.learning_object.review_average}
                                                        author={card.submitter.name}
                                                        tags={card.learning_object.tags}
                                                        href={"/recurso/" + card.learning_object.id}
                                                        downloadableLink={card.default_attachment_location}
                                                    />
                                                </Grid>
                                            )
                                        }
                                    </StyledGrid>
                                    {
                                        props.loadingMore ?
                                            <LoadingSpinner contrast={props.contrast} text="Carregando recurso..." />
                                            :
                                            <ButtonsAreaRecurso
                                                contrast={props.contrast}
                                                sliceLength={props.sliceArr.length}
                                                length={props.length}
                                                showMore={props.showMore}
                                                end={String(props.sliceArr.length) === props.end || Number(props.sliceArr.length) === props.end}
                                                total={props.end}
                                            />
                                    }

                                </React.Fragment>
                            ]
                        )
            }

        </WhiteContainer>
    )
}
