/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useContext } from 'react'
import { Store } from '../../../Store.js';
import styled from 'styled-components'
import Button from '@material-ui/core/Button';
import { Link } from 'react-router-dom';

//Image Import
import { Publicar } from "ImportImages.js";

//3 casos
//Professor nao cadastrado (nao pediu OU submitter_request = rejected)
//Professor em análise (submitter_request = requested)
//Professor aceito (submitter_request = accepted)
export default function TabPanelSolicitarContaProfessor(props) {
    // eslint-disable-next-line
    const { state, dispatch } = useContext(Store)

    return (
        <div className='card-config'>
            <div className='content-div'>
                {
                    !state.currentUser.roles.some(role => role.name === "teacher") && (state.currentUser.submitter_request === 'default') ?
                        (
                            <div>
                                <ImageDiv />
                                <StyledH2 contrast={state.contrast}>Você é professor(a) da educação básica e gostaria de colaborar com a Plataforma?</StyledH2>
                                <StyledP contrast={state.contrast}>Ao ser identificado como professor(a), você poderá publicar e compartilhar recursos educacionais
                                digitais na plataforma com toda a comunidade escolar do país.
                                </StyledP>
                                <div>
                                    <Link to={{
                                        pathname: '/termos-publicar-recurso',
                                        state: true
                                    }}>
                                        <CompletarCadastroButton contrast={state.contrast}>
                                            SIM, COMPLETAR CADASTRO
                                        </CompletarCadastroButton>
                                    </Link>
                                </div>
                            </div>
                        )
                        :
                        (
                            <>
                                {
                                    (() => {
                                        switch (state.currentUser.submitter_request) {
                                            case 'requested':
                                                return (
                                                    <h1 className="title">A sua conta de Professor foi solicitada</h1>
                                                )
                                            case 'rejected':
                                                return (
                                                    <h1 className="title">Sua solicitação foi negada</h1>
                                                )
                                            default:
                                                return (
                                                    <h1 className="title">Você já possui uma conta de Professor</h1>
                                                )
                                            
                                        }
                                    })()
                                }
                            </>
                        )
                }
            </div>
        </div>
    )
}

export const CompletarCadastroButton = styled(Button)`
    background-color: ${props => props.contrast === "" ? "#00bcd4 !important" : "black !important"};
    color: ${props => props.contrast === "" ? "#fff !important" : "yellow !important"};
    font-family : 'Roboto',sans-serif !important;
    font-size : 14px !important;
    font-weight : bold !important;
    height : 36px !important;
    border-radius : 3px !important;
    padding-left : 16px !important;
    padding-right : 16px !important;
    outline : none !important;
    margin : 6px 8px !important;
    white-space : nowrap !important;
    text-transform : uppercase !important;
    font-weight : bold !important;
    font-size : 14px !important;
    font-style : inherit !important;
    font-variant : inherit !important;
    font-family : inherit !important;
    text-decoration: ${props => props.contrast === "" ? "none !important" : "underline !important"};
    overflow : hidden !important;
    display : inline-block !important;
    position : relative !important;
    cursor : pointer !important;
    min-height : 36px !important;
    min-width : 88px !important;
    vertical-align : middle !important;
    align-items : center !important;
    text-align : center !important;
    box-sizing : border-box !important;
    border: ${props => props.contrast === "" ? "0 !important" : "1px solid white !important"};
    :hover{
        background-color: ${props => props.contrast === "" ? "" : "rgba(255,255,0,0.24) !important"};
    }
`

const StyledP = styled.p`
    font-size : 15px;
    line-height : 22px;
    text-align : left;
    padding-bottom : 20px;
    color: ${props => props.contrast === "" ? "" : "white"};
`

const StyledH2 = styled.h2`
    color: ${props => props.contrast === "" ? "" : "white"};
    font-weight : 200;
    font-size : 26px;
    line-height : 36px;
    text-align : left;
    padding-bottom : 20px;
`

const ImageDiv = styled.div`
    background-image : url(${Publicar});
    display : block;
    height : 114px;
    background-size : contain;
    background-position : center center;
    background-repeat : no-repeat;
`
